/*
   Suppose you want to rotate a 2D model around the point (a,b) in
   the model's coordinates, and you want this rotation to happen at
   the point (c,d) in the camera's view coordinates. You would do
   that with the following three transformations.

      glLoadIdentity()
      glTranslate(c, d, 0);
      glRotate(theta, 0, 0, 1);
      glTranslate(-a, -b, 0);

      M = I * T(c,d) * R * T(-a,-b)

      for ()
         create vertex v
         apply matrix M
*/

import com.jogamp.opengl.GLEventListener;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.awt.GLJPanel;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.glu.GLU;
import com.jogamp.opengl.util.FPSAnimator;

import javax.swing.JFrame;

public class Square2 implements GLEventListener {

   private int frameNumber = 0;

   @Override
   public void init(GLAutoDrawable drawable)
   {
    //drawable.setGL(new com.jogamp.opengl.TraceGL2((GL2)drawable.getGL(), System.err));

      final GL2 gl = drawable.getGL().getGL2();
      final GLU glu = new GLU();  // get GL Utilities

      // Initialize OpenGL state
      // attributes
      gl.glClearColor(1.0f, 1.0f, 1.0f, 1.0f); // white background

      // set up viewing
      gl.glMatrixMode(GL2.GL_PROJECTION);
      gl.glLoadIdentity();
      gl.glOrtho(-1.0f, 6.0f, 0.0f, 2.0f, -1.0f, 20f); // orthographic viewing volume
      //glu.gluPerspective(90, 1.0, 0.1, 20); // perspective viewing volume
      gl.glMatrixMode(GL2.GL_MODELVIEW);
   }

   @Override
   public void display(GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();

      gl.glClear(GL2.GL_COLOR_BUFFER_BIT); // clear the graphics panel

      gl.glLoadIdentity();
      // Draw an xy-axis.
      gl.glLoadIdentity();
      gl.glColor3d(0.0, 0.0, 0.0);
      gl.glBegin(GL2.GL_LINES);
        gl.glVertex3d(-1.8,  0.0, 0.0);
        gl.glVertex3d( 1.8,  0.0, 0.0);
        gl.glVertex3d( 0.0, -1.8, 0.0);
        gl.glVertex3d( 0.0,  1.8, 0.0);
      gl.glEnd();

      if (frameNumber < 90)
      {
         gl.glLoadIdentity();
         gl.glTranslated( 1.0,  0.0, 0.0);  //  (c, d)
         gl.glRotated(-frameNumber, 0.0f, 0.0f, 1.0f);
         gl.glTranslated(-1.0, 0.0, 0.0);  // (-a, -b)
      }
      else if (frameNumber < 180)
      {
        gl.glLoadIdentity();
          gl.glTranslated( 2.0,  0.0, 0.0);  //  (c, d)
          gl.glRotated(-frameNumber, 0.0f, 0.0f, 1.0f);
          gl.glTranslated( -1.0, -1.0, 0.0);  // (-a, -b)
     }
      else if (frameNumber < 270)
      {
        gl.glLoadIdentity();
          gl.glTranslated( 3.0,  0.0, 0.0);  //  (c, d)
          gl.glRotated(-frameNumber, 0.0f, 0.0f, 1.0f);
          gl.glTranslated( 0.0, -1.0, 0.0);  // (-a, -b)
     }
      else if (frameNumber < 360)
      {
        gl.glLoadIdentity();
          gl.glTranslated( 4.0,  0.0, 0.0);  //  (c, d)
          gl.glRotated(-frameNumber, 0.0f, 0.0f, 1.0f);
          gl.glTranslated( 0.0, 0.0, 0.0);  // (-a, -b)
     }
      drawSquare(drawable);

      frameNumber = (frameNumber + 1) % 360; // one degre per frame
   }

   private static void drawSquare(GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();
      gl.glBegin(GL2.GL_TRIANGLES);
         gl.glColor3f( 1.0f, 0.0f, 0.0f);
         gl.glVertex3f(0.0f, 0.0f, 0.0f);
         gl.glColor3f( 0.0f, 1.0f, 0.0f);
         gl.glVertex3f(0.0f, 1.0f, 0.0f);
         gl.glColor3f( 1.0f, 1.0f, 0.0f);
         gl.glVertex3f(1.0f, 0.0f, 0.0f);

         gl.glColor3f( 1.0f, 1.0f, 0.0f);
         gl.glVertex3f(1.0f, 0.0f, 0.0f);
         gl.glColor3f( 0.0f, 1.0f, 0.0f);
         gl.glVertex3f(0.0f, 1.0f, 0.0f);
         gl.glColor3f( 0.0f, 0.0f, 1.0f);
         gl.glVertex3f(1.0f, 1.0f, 0.0f);
      gl.glEnd();
      gl.glFlush(); // flush the buffers
   }


   @Override
   public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h){}

   @Override
   public void dispose(GLAutoDrawable drawable){}


   public static void main(String[] args)
   {
      // Get a capabilities object from a GL2 profile.
      final GLProfile glProfile = GLProfile.get(GLProfile.GL2);
      final GLCapabilities glCapabilities = new GLCapabilities(glProfile);

      // Use it to create a GLJpanel.
      final GLJPanel glDrawable = new GLJPanel(glCapabilities);
      glDrawable.setPreferredSize(new java.awt.Dimension(700, 200));

      // Connect the GLJPanel with our GLEventListener implementation.
      glDrawable.addGLEventListener( new Square2() );

      // Create a JFrame.
      final JFrame jFrame = new JFrame("Square 2");
      jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

      // Add the GLJPanel to the JFrame.
      jFrame.getContentPane().add(glDrawable);
      jFrame.pack();
      jFrame.setLocationRelativeTo(null);
      jFrame.setVisible(true);

      // Create an animation thread.
      final FPSAnimator animator = new FPSAnimator(glDrawable, 15, true);
      animator.start();
   }
}
