/*
   Use this program to experiment with two-dimensional transformations.
*/

import com.jogamp.opengl.GLEventListener;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.awt.GLJPanel;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.glu.GLU;
import com.jogamp.opengl.util.FPSAnimator;

import javax.swing.JFrame;

public class Transformations implements GLEventListener {

   private float  theta = 0.0f;  // degrees
   private float dTheta = 2.0f;  // degrees
   private float  z = -0.2f;
   private float dz = -0.01f;

   @Override
   public void init(GLAutoDrawable drawable)
   {
    //drawable.setGL(new com.jogamp.opengl.TraceGL2((GL2)drawable.getGL(), System.err));

      final GL2 gl = drawable.getGL().getGL2();
      final GLU glu = new GLU();  // get GL Utilities

      // Initialize OpenGL state
      // attributes
      gl.glClearColor(1.0f, 1.0f, 1.0f, 1.0f); // white background

      // set up viewing
      gl.glMatrixMode(GL2.GL_PROJECTION);
      gl.glLoadIdentity();
    //gl.glOrtho(-1.0f, 1.0f, -1.0f, 1.0f, -1.0f, 20f); // orthographic viewing volume
      glu.gluPerspective(90, 1.0, 0.1, 20); // perspective viewing volume
      gl.glMatrixMode(GL2.GL_MODELVIEW);
   }

   @Override
   public void display(GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();

      gl.glClear(GL2.GL_COLOR_BUFFER_BIT); // clear the graphics panel

      gl.glRotated(dTheta, 0.0f, 0.0f, 1.0f); // accumulate rotations
      gl.glTranslated(0.0, 0.0, dz);          // accumulte translations

      double theta1 =   0 *Math.PI/180,
             theta2 = 120 *Math.PI/180,
             theta3 = 240 *Math.PI/180;
      gl.glBegin(GL2.GL_TRIANGLES); // draw a triangle
         gl.glColor3f(1.0f, 0.0f, 0.0f);
         gl.glVertex3d(Math.cos(theta1), Math.sin(theta1), -0.2);
         gl.glColor3f(0.0f, 1.0f, 0.0f);
         gl.glVertex3d(Math.cos(theta2), Math.sin(theta2), -0.2);
         gl.glColor3f(0.0f, 0.0f, 1.0f);
         gl.glVertex3d(Math.cos(theta3), Math.sin(theta3), -0.2);
      gl.glEnd();
      gl.glFlush(); // flush the buffers

      //theta += dTheta;
      //z += dz;
   }


   @Override
   public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h){}

   @Override
   public void dispose(GLAutoDrawable drawable){}


   public static void main(String[] args)
   {
      // Get a capabilities object from a GL2 profile.
      final GLProfile glProfile = GLProfile.get(GLProfile.GL2);
      final GLCapabilities glCapabilities = new GLCapabilities(glProfile);

      // Use it to create a GLJpanel.
      final GLJPanel glDrawable = new GLJPanel(glCapabilities);
      glDrawable.setPreferredSize(new java.awt.Dimension(400, 400));

      // Connect the GLJPanel with our GLEventListener implementation.
      glDrawable.addGLEventListener( new Transformations() );

      // Create a JFrame.
      final JFrame jFrame = new JFrame("Animation");
      jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

      // Add the GLJPanel to the JFrame.
      jFrame.getContentPane().add(glDrawable);
      jFrame.pack();
      jFrame.setLocationRelativeTo(null);
      jFrame.setVisible(true);

      // Create an animation thread.
      final FPSAnimator animator = new FPSAnimator(glDrawable, 24, true);
      animator.start();
   }
}
